@@ -142,10 +142,14 @@ class DoubleBufferedIterator(Iterator[T]):
     Note: The inner iterable should *not* return None"""
 
     def __init__(self, original_iterable: Iterable[T], max_queue_size_inner: int=20, max_queue_size_outer: int=5):
+        def get_inner_iterable():
+            return self.__worker_inner(original_iterable)
+        def get_outer_iterable():
+            return self.__worker_outer()
         self.__buffer_inner = multiprocessing.Queue(maxsize=max_queue_size_inner)  # type: multiprocessing.Queue[Union[None, T, Tuple[Exception, Any]]]
         self.__buffer_outer = multiprocessing.Queue(maxsize=max_queue_size_outer)  # type: multiprocessing.Queue[Union[None, T, Tuple[Exception, Any]]]
-        self.__worker_process_inner = multiprocessing.Process(target=lambda: self.__worker_inner(original_iterable))
-        self.__worker_process_outer = multiprocessing.Process(target=lambda: self.__worker_outer())
+        self.__worker_process_inner = multiprocessing.Process(target=get_inner_iterable)
+        self.__worker_process_outer = multiprocessing.Process(target=get_outer_iterable)
         self.__worker_process_inner.start()
         self.__worker_process_outer.start()
 
@@ -192,26 +196,30 @@ class DoubleBufferedIterator(Iterator[T]):
         return next_element
 
 
-def shuffled_iterator(input_iterator: Iterator[T], buffer_size: int = 10000, out_slice_sizes: int = 500) -> Iterator[T]:
+def shuffled_iterator(input_iterator: Iterator[T], buffer_size: int = 10000, rng: Optional[random.Random]=None) -> Iterator[T]:
     """
     Accept an iterator and return an approximate streaming (and memory efficient) shuffled iterator.
 
-    To achieve (approximate) shuffling a buffer of elements is stored. Once the buffer is full, it is shuffled and
-    `out_slice_sizes` random elements from the buffer are returned. Thus, there is a good bias for
-    yielding the first set of elements in input early.
+    To achieve (approximate) shuffling a buffer of elements is stored. Once the buffer is full, it is shuffled
+    and random elements are yielded from the buffer, while it continues to be replenished.
+
+    Notes:
+         * There is a good bias for yielding the first set of elements in input early.
+         * There is a delay for this wrapper to yield elements as the buffer needs to be filled in first
+            with `buffer_size` elements or the `input_iterator` to be exhausted.
 
     """
-    assert out_slice_sizes <= buffer_size, 'out_slices_size cannot be larger than buffer_size.'
-
+    if rng is None:
+        rng = random
     # Ensure that this is an iterator that can be consumed exactly once.
     input_iterator = iter(input_iterator)
 
     buffer = list(islice(input_iterator, buffer_size))  # type: List[T]
-    random.shuffle(buffer)
+    rng.shuffle(buffer)
 
     for element in input_iterator:
         # Pick a random element in the buffer to yield and replace it with a new element
-        idx = random.randrange(buffer_size)
+        idx = rng.randrange(buffer_size)
         to_yield, buffer[idx] = buffer[idx], element
         yield to_yield
 
